/****************************************************************************
** Meta object code from reading C++ file 'providermanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.9.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/attica-5.43.0/src/providermanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'providermanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.9.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Attica__ProviderManager_t {
    QByteArrayData data[18];
    char stringdata0[286];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Attica__ProviderManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Attica__ProviderManager_t qt_meta_stringdata_Attica__ProviderManager = {
    {
QT_MOC_LITERAL(0, 0, 23), // "Attica::ProviderManager"
QT_MOC_LITERAL(1, 24, 13), // "providerAdded"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 16), // "Attica::Provider"
QT_MOC_LITERAL(4, 56, 8), // "provider"
QT_MOC_LITERAL(5, 65, 22), // "defaultProvidersLoaded"
QT_MOC_LITERAL(6, 88, 32), // "authenticationCredentialsMissing"
QT_MOC_LITERAL(7, 121, 8), // "Provider"
QT_MOC_LITERAL(8, 130, 12), // "fileFinished"
QT_MOC_LITERAL(9, 143, 3), // "url"
QT_MOC_LITERAL(10, 147, 12), // "authenticate"
QT_MOC_LITERAL(11, 160, 14), // "QNetworkReply*"
QT_MOC_LITERAL(12, 175, 15), // "QAuthenticator*"
QT_MOC_LITERAL(13, 191, 27), // "proxyAuthenticationRequired"
QT_MOC_LITERAL(14, 219, 13), // "QNetworkProxy"
QT_MOC_LITERAL(15, 233, 5), // "proxy"
QT_MOC_LITERAL(16, 239, 13), // "authenticator"
QT_MOC_LITERAL(17, 253, 32) // "slotLoadDefaultProvidersInternal"

    },
    "Attica::ProviderManager\0providerAdded\0"
    "\0Attica::Provider\0provider\0"
    "defaultProvidersLoaded\0"
    "authenticationCredentialsMissing\0"
    "Provider\0fileFinished\0url\0authenticate\0"
    "QNetworkReply*\0QAuthenticator*\0"
    "proxyAuthenticationRequired\0QNetworkProxy\0"
    "proxy\0authenticator\0"
    "slotLoadDefaultProvidersInternal"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Attica__ProviderManager[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,
       5,    0,   52,    2, 0x06 /* Public */,
       6,    1,   53,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    1,   56,    2, 0x08 /* Private */,
      10,    2,   59,    2, 0x08 /* Private */,
      13,    2,   64,    2, 0x08 /* Private */,
      17,    0,   69,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7,    4,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, 0x80000000 | 11, 0x80000000 | 12,    2,    2,
    QMetaType::Void, 0x80000000 | 14, 0x80000000 | 12,   15,   16,
    QMetaType::Void,

       0        // eod
};

void Attica::ProviderManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        ProviderManager *_t = static_cast<ProviderManager *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->providerAdded((*reinterpret_cast< const Attica::Provider(*)>(_a[1]))); break;
        case 1: _t->defaultProvidersLoaded(); break;
        case 2: _t->authenticationCredentialsMissing((*reinterpret_cast< const Provider(*)>(_a[1]))); break;
        case 3: _t->fileFinished((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->authenticate((*reinterpret_cast< QNetworkReply*(*)>(_a[1])),(*reinterpret_cast< QAuthenticator*(*)>(_a[2]))); break;
        case 5: _t->proxyAuthenticationRequired((*reinterpret_cast< const QNetworkProxy(*)>(_a[1])),(*reinterpret_cast< QAuthenticator*(*)>(_a[2]))); break;
        case 6: _t->slotLoadDefaultProvidersInternal(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            typedef void (ProviderManager::*_t)(const Attica::Provider & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ProviderManager::providerAdded)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (ProviderManager::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ProviderManager::defaultProvidersLoaded)) {
                *result = 1;
                return;
            }
        }
        {
            typedef void (ProviderManager::*_t)(const Provider & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ProviderManager::authenticationCredentialsMissing)) {
                *result = 2;
                return;
            }
        }
    }
}

const QMetaObject Attica::ProviderManager::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_Attica__ProviderManager.data,
      qt_meta_data_Attica__ProviderManager,  qt_static_metacall, nullptr, nullptr}
};


const QMetaObject *Attica::ProviderManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Attica::ProviderManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Attica__ProviderManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Attica::ProviderManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void Attica::ProviderManager::providerAdded(const Attica::Provider & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Attica::ProviderManager::defaultProvidersLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Attica::ProviderManager::authenticationCredentialsMissing(const Provider & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
