/****************************************************************************
** Meta object code from reading C++ file 'atticabasejob.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.9.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/attica-5.43.0/src/atticabasejob.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'atticabasejob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.9.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Attica__BaseJob_t {
    QByteArrayData data[12];
    char stringdata0[133];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Attica__BaseJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Attica__BaseJob_t qt_meta_stringdata_Attica__BaseJob = {
    {
QT_MOC_LITERAL(0, 0, 15), // "Attica::BaseJob"
QT_MOC_LITERAL(1, 16, 8), // "finished"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 16), // "Attica::BaseJob*"
QT_MOC_LITERAL(4, 43, 3), // "job"
QT_MOC_LITERAL(5, 47, 5), // "start"
QT_MOC_LITERAL(6, 53, 5), // "abort"
QT_MOC_LITERAL(7, 59, 12), // "dataFinished"
QT_MOC_LITERAL(8, 72, 6), // "doWork"
QT_MOC_LITERAL(9, 79, 22), // "authenticationRequired"
QT_MOC_LITERAL(10, 102, 14), // "QNetworkReply*"
QT_MOC_LITERAL(11, 117, 15) // "QAuthenticator*"

    },
    "Attica::BaseJob\0finished\0\0Attica::BaseJob*\0"
    "job\0start\0abort\0dataFinished\0doWork\0"
    "authenticationRequired\0QNetworkReply*\0"
    "QAuthenticator*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Attica__BaseJob[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   47,    2, 0x0a /* Public */,
       6,    0,   48,    2, 0x0a /* Public */,
       7,    0,   49,    2, 0x09 /* Protected */,
       8,    0,   50,    2, 0x08 /* Private */,
       9,    2,   51,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10, 0x80000000 | 11,    2,    2,

       0        // eod
};

void Attica::BaseJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        BaseJob *_t = static_cast<BaseJob *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->finished((*reinterpret_cast< Attica::BaseJob*(*)>(_a[1]))); break;
        case 1: _t->start(); break;
        case 2: _t->abort(); break;
        case 3: _t->dataFinished(); break;
        case 4: _t->doWork(); break;
        case 5: _t->authenticationRequired((*reinterpret_cast< QNetworkReply*(*)>(_a[1])),(*reinterpret_cast< QAuthenticator*(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Attica::BaseJob* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            typedef void (BaseJob::*_t)(Attica::BaseJob * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BaseJob::finished)) {
                *result = 0;
                return;
            }
        }
    }
}

const QMetaObject Attica::BaseJob::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_Attica__BaseJob.data,
      qt_meta_data_Attica__BaseJob,  qt_static_metacall, nullptr, nullptr}
};


const QMetaObject *Attica::BaseJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Attica::BaseJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Attica__BaseJob.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Attica::BaseJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Attica::BaseJob::finished(Attica::BaseJob * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
