// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "attica_debug.h"


#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
Q_LOGGING_CATEGORY(ATTICA, "org.kde.attica", QtInfoMsg)
#else
Q_LOGGING_CATEGORY(ATTICA, "org.kde.attica")
#endif

